let AWS = require('aws-sdk');
let now = new Date();
let fs = require('fs');
let S3Wrapper = require('../lib/S3Wrapper');
let InstanceItem = require('../lib/index');

function LaunchError(error) {
    this.error = error;
    this._at = new Date();
    this._id = (require('node-uuid')).v4()
}

let LaunchScript = async function (instance, next) {

    try {
        instance.providers = await GetSetupProviders();
        if (!instance.providers.location_id) throw "Provider ERROR - No LocationId Found In Provider";
        let LocalS3 = new S3Wrapper(new AWS.S3({apiVersion : "2006-03-01", region: instance.region}), {
            Bucket: instance.providers.machine_tag+'-configurations',
            Key : 'instances_versions/'+instance.providers.machine_tag_u+'_location_creds.json',
        });
        let config = await LocalS3.get();
        if (!config) return console.log('No Config Returned');
        instance.root_dynamo = new AWS.DynamoDB({
            apiVersion: "2012-08-10",
            region: instance.region,
            accessKeyId: config.credentials.outer_access.root.accessId,
            secretAccessKey: config.credentials.outer_access.root.secretKey
        });
        let root = await GetRootDynamo(instance.providers.tablename, instance.providers.location_id);
        return Promise.resolve(root);
    } catch(e) {
        return Promise.reject(e);
    }

    function GetRootDynamo(tablename, location) {
        return new Promise(function (result) {
            let instance_id = (!instance.Instance) ? ((!instance.instance_id) ? null : instance.instance_id) : instance.Instance.instance_id;
            if (!instance_id) return result(Promise.reject("No Instance ID Present In Object"));
            let query = {
                Key: {
                    location_id: {
                        S: location
                    },
                    instance_id: {
                        S: instance_id
                    }
                },
                TableName: tablename,
                AttributesToGet : [
                    "instance_id",
                    "location_id"
                ]
            };
            instance.root_dynamo.getItem(query, function (error, done) {
                if (error) return result(Promise.reject(error));
                else {
                    let doing = require('./cleanup'), clean = new doing();
                    clean.done(instance.instance_id);
                    if (done.Item && Object.keys(done.Item).length > 0) {
                        instance.root_instance_params  = new InstanceItem.InstanceItem();
                        instance.root_instance_params.addDynamoValues(done.Item).then(function () {
                            return result(Promise.resolve());
                        }).catch(function (err) {
                            return result(Promise.reject(new LaunchError(err)))
                        });
                    } else {
                        return result(Promise.resolve(new InstanceItem.None(query)));
                    }
                }
            });
        })
    }

//TODO: RENAME next.json to setup.json
    function GetSetupProviders() {
        return new Promise(function (result) {
            if (next) {
                if (!next.machine_tag) return result(Promise.reject('Failed To Launch'+"Invalid Next File - No Machine Tag"));
                if (!next.machine_tag_u) return result(Promise.reject('Failed To Launch'+"Invalid Next File - No Machine Tag U"));
                if (!next.tablename) throw "Invalid - Misssing TableName From Providers";
                instance.providers = next;
                return result(Promise.resolve(next));
            } else {
                fs.readFile("../../../next.json", 'utf8', async function (err, cont) {
                    if (err) return result(Promise.reject(err));
                    try {
                        let json = await JSON.parse(cont);
                        if (!json.machine_tag) return result(Promise.reject('Failed To Launch'+"Invalid Next File - No Machine Tag"));
                        if (!json.machine_tag_u) return result(Promise.reject('Failed To Launch'+"Invalid Next File - No Machine Tag U"));
                        if (!json.tablename) throw "Invalid - Misssing TableName From Providers";
                        instance.providers = json;
                        return result(Promise.resolve(json));
                    } catch(e) {
                        return result(Promise.reject(e));
                    }
                })
            }
        })
    }
};

module.exports = LaunchScript;