
let S3Wrapper = require('../lib/S3Wrapper');
let axios = require('axios');

module.exports = async function AlertDispatcher(instance) {
    let region = process.env['region'];
    let instanceid = process.env['instance_id'];
    if (!region || !instanceid) throw "Missing Basic Information";
    instanceid = instanceid.replace('-',"_");
    try {
        if (process.env.DEBUG_TESTER && process.env.DEBUG_TESTER === process.env.region) return Promise.resolve();
        let Machine = new S3Wrapper(null, {
            Bucket: 'locality-pos',
            Key: 'region-details.json'});
        let MachineJSON = await Machine.get();
        let Wrap = new S3Wrapper(null, {
            Bucket: MachineJSON.region_name+'-dispatcher-metadata',
            Key: MachineJSON.region_name+'-dispatcher-notifiers-creds.json'});
        let json = await Wrap.get();
        if (!json.path) throw "Missing Path";
        if (!json.authorization) throw "Missing Auth";
        if (!json.accessor) throw "Missing Accessor";
        let locid = (!process.env.LOCATION_ID) ? await GetLocID(instanceid) : process.env.LOCATION_ID;
        let request = await axios.post(json.path, {
            account: MachineJSON.region_name,
            authorization: json.authorization,
            message: "instance="+process.env['instance_id']+"&action=launched&port=3000&location="+locid
        }, {
            headers: {
                'x-api-key': json.accessor,
                "Content-Type":"application/json"
            }});
        if (!request.data) throw "Error : "+"Request failed";
        console.log(request.data.error);
        console.log(request.data.body);
        let code = request.data.body.statusCode;
        if (code !== 200) throw "Error : "+request.data.body.message;
        return Promise.resolve();
    } catch(e) {
        throw e;
    }
    function GetLocID(insid) {
        let AWS = require('aws-sdk');
        let dynamo = new AWS.DynamoDB({apiVersion: "2012-08-10", region: region});
        return new Promise(function (result) {
            dynamo.getItem({
                TableName: "location_information",
                Key: {
                    "instance_id": {
                        S: insid
                    }
                }
            }, function (err, data) {
                if (err) return result(Promise.reject(err));
                if (!data.Item) return result(Promise.reject("Invalid Item Found For Location"));
                if (!data.Item.location_id || !data.Item.location_id.S) return result(Promise.reject("Invalid Item Found - No Location"));
                process.env.LOCATION_ID = data.Item.location_id.S;
                return result(data.Item.location_id.S);
            })
        })
    }
}